from pyspark_easy.utils import *

from sklearn.metrics import confusion_matrix

from sklearn.metrics import accuracy_score, precision_score, recall_score, f1_score, roc_auc_score

import sklearn.metrics as metrics

from texttable import Texttable


class pyspark_model_eval():

    def __init__(self, model, predicted_df):

        if 'pyspark.ml.classification' in str(type(model)):

            self.type_of_model = model.numClasses

            self.label_column = model.summary.labelCol

            self.y_true = predicted_df.select([self.label_column]).rdd.flatMap(lambda x: x).collect()

            self.y_pred = predicted_df.select(['prediction']).rdd.flatMap(lambda x: x).collect()

            self.yp_pred = predicted_df.select(['probability']).rdd.flatMap(lambda x: x).collect()

            self.cm = confusion_matrix(self.y_true, self.y_pred)

            self.model = model

        else:

            print("This is not a Classification Model")

    def results(self):

        t = Texttable()

        train_test = []

        train_test.append(['Metrics', 'Train', 'Test'])

        if hasattr(self.model.summary, 'areaUnderROC'):
            train_roc = self.model.summary.areaUnderROC

            test_roc = roc_auc_score(self.y_true, self.y_pred)

            train_test.append(['AreaUnderROC', round(train_roc * 100, 2), round(test_roc * 100, 2)])

        train_accuracy = self.model.summary.accuracy

        test_accuracy = accuracy_score(self.y_true, self.y_pred)

        train_test.append(['Accuracy', round(train_accuracy * 100, 2), round(test_accuracy * 100, 2)])

        train_precision = self.model.summary.weightedPrecision

        test_precision = precision_score(self.y_true, self.y_pred, average='weighted')

        train_test.append(['Weighted Precision', round(train_precision * 100, 2), round(test_precision * 100, 2)])

        train_recall = self.model.summary.weightedRecall

        test_recall = recall_score(self.y_true, self.y_pred, average='weighted')

        train_test.append(['Weighted Recall', round(train_recall * 100, 2), round(test_recall * 100, 2)])

        train_f1score = self.model.summary.weightedFMeasure()

        test_f1score = f1_score(self.y_true, self.y_pred, average='weighted')

        train_test.append(['Weighted F1 Score', round(train_f1score * 100, 2), round(test_f1score * 100, 2)])

        t.add_rows(train_test)

        print("The Train vs Test evaluation Metrics are :")

        print(t.draw())

        print("Note: The Weighted metrics - calculated by label then it is averaged")

        plot_cm(self.cm)

        if hasattr(self.model.summary, 'roc'):
            roc = self.model.summary.roc

            training_roc(roc)

        if hasattr(self.model.summary, 'pr'):
            pr = self.model.summary.pr

            training_pr(pr)

        if self.type_of_model == 2:
            fpr, tpr, threshold = metrics.roc_curve(self.y_true, self.y_pred)

            roc_auc = metrics.auc(fpr, tpr)

            ROC(fpr, tpr, roc_auc)

            precision, recall, thresholds = metrics.precision_recall_curve(self.y_true, self.y_pred)

            auc = metrics.auc(recall, precision)

            PR(recall, precision, auc)

        if self.type_of_model > 2:

            classes = np.unique(self.y_true)

            i = 0

            j = 5

            while True:

                plt_roc(self.y_true, self.yp_pred, classes=classes[i:j])

                plt_pr(self.y_true, self.yp_pred, classes=classes[i:j])

                if j > len(classes):

                    break

                else:

                    i = i + 5

                    j = j + 5

        else:

            plt_roc(self.y_true, self.yp_pred)

            plt_pr(self.y_true, self.yp_pred)

        if self.type_of_model == 2:
            plt_lift(self.y_true, self.yp_pred)

            plt_cumulative(self.y_true, self.yp_pred)

            t = Texttable()

            train_test = []

            train_test.append(['Metrics', 'Test Data'])

            precision = precision_score(self.y_true, self.y_pred)

            train_test.append(['Precision', round(precision * 100, 2)])

            recall = recall_score(self.y_true, self.y_pred)

            train_test.append(['Recall', round(recall * 100, 2)])

            f_score = f1_score(self.y_true, self.y_pred)

            train_test.append(['F1 Score', round(f_score * 100, 2)])

            t.add_rows(train_test)

            print("The binary classification metrics for test data are:")

            print(t.draw())

            print("\n")

        print("The classification report follows :")

        return classif_report(self.y_true, self.y_pred)


###########################################################
import numpy as np

import seaborn as sns

import matplotlib.pyplot as plt

import scikitplot as skplt

import pandas as pd

from sklearn.metrics import classification_report

#Util functions 

def Extract(lst): 
    return [item[0] for item in lst]


fontdict = {'fontsize': 9, 'fontweight': 'medium'}


def training_roc(roc):
    roc = roc.toPandas()

    plt.plot(roc['FPR'], roc['TPR'])

    plt.ylabel('False Positive Rate', fontdict=fontdict, color='black')

    plt.xlabel('True Positive Rate', fontdict=fontdict, color='black')

    plt.title('Training - ROC Curve', fontsize=10, weight='bold', color='steelblue')

    plt.plot([0, 1], [0, 1], 'k--')

    plt.xticks([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0], fontsize=8, weight='bold', color='darkslategrey')

    plt.yticks([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0], fontsize=8, weight='bold', color='darkslategrey')

    plt.show()


def training_pr(pr):
    pr = pr.toPandas()

    plt.plot(pr['recall'], pr['precision'])

    plt.ylabel('Precision', fontdict=fontdict, color='black')

    plt.xlabel('Recall', fontdict=fontdict, color='black')

    plt.title('Training - PR Curve', fontsize=10, weight='bold', color='steelblue')

    plt.plot([0, 1], [0, 1], 'r--')

    plt.xticks([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0], fontsize=8, weight='bold', color='darkslategrey')

    plt.yticks([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0], fontsize=8, weight='bold', color='darkslategrey')

    plt.show()


def plot_cm(cm):

 fig_len = 1 + cm.shape[0]

 fig_wid = round(fig_len * 0.75)

 fig, ax = plt.subplots(figsize=(fig_len, fig_wid))

 sns.set(font_scale=0.75)

 cm_sum = np.sum(cm, axis=1, keepdims=True)

 cm_perc = cm / cm_sum.astype(float) * 100

 annot = np.empty_like(cm).astype(str)

 nrows, ncols = cm.shape

 for i in range(nrows):

    for j in range(ncols):

        c = cm[i, j]

        p = cm_perc[i, j]

        if i == j:

            s = cm_sum[i]

            annot[i, j] = '%d\n\n(%.1f%%)' % (c, p)

        elif c == 0:

            annot[i, j] = '%d' % c

        else:

            annot[i, j] = '%d\n\n(%.1f%%)' % (c, p)

 sns.heatmap(cm, annot=annot, cmap='Blues', xticklabels='auto', cbar=True,
            annot_kws={"fontsize": 8, 'fontweight': 'medium'}, fmt='', ax=ax);  # annot=True to annotate cells

# labels, title and ticks

 ax.set_xlabel('Predicted labels', fontdict=fontdict, color='slategrey')

 ax.set_ylabel('True labels', fontdict=fontdict, color='slategrey')

 ax.set_title('Confusion Matrix', fontsize=10, weight='bold', color='steelblue')

 plt.xticks(fontsize=8, weight='bold', color='darkslategrey')

 plt.yticks(fontsize=8, weight='bold', color='darkslategrey')

 plt.show()


def classif_report(y_true, y_pred):
    df = pd.DataFrame(classification_report(y_true, y_pred,

                                            output_dict=True)).T

    for i in df.columns:
        df[i] = df[i].round(decimals=2)

    df['support'] = df.support.apply(int)

    return df.style.background_gradient(cmap='YlGn',

                                        subset=pd.IndexSlice[df.index[:-3], :'f1-score'])


def plt_lift(y_true, yp_pred):

 skplt.metrics.plot_lift_curve(y_true, yp_pred)

 plt.title('Lift Curve', fontsize=10, weight='bold', color='steelblue')

 plt.ylabel('Lift', fontdict=fontdict, color='black')

 plt.xlabel('Percentage of Sample', fontdict=fontdict, color='black')

 plt.show()


def plt_cumulative(y_true, yp_pred):

 skplt.metrics.plot_cumulative_gain(y_true, yp_pred)

 plt.title('Cumulative Gains Curve', fontsize=10, weight='bold', color='steelblue')

 plt.ylabel('Gain', fontdict=fontdict, color='black')

 plt.xlabel('Percentage of Sample', fontdict=fontdict, color='black')

 plt.show()


def plt_roc(y_true, yp_pred, classes=None):

 skplt.metrics.plot_roc(y_true, yp_pred, classes_to_plot=classes, plot_micro=False, plot_macro=False)

 plt.title('Test Data - Receiver Operating Characteristic by Class', fontsize=10, weight='bold', color='steelblue')

 plt.xticks([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0], fontsize=8, weight='bold', color='darkslategrey')

 plt.yticks([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0], fontsize=8, weight='bold', color='darkslategrey')

 plt.show()


def plt_pr(y_true, yp_pred, classes=None):

 skplt.metrics.plot_precision_recall(y_true, yp_pred, classes_to_plot=classes, plot_micro=False)

 plt.plot([0, 1], [0, 1], 'k--')

 plt.title('Test Data - Precision-Recall Curve by Class', fontsize=10, weight='bold', color='steelblue')

 plt.xticks([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0], fontsize=8, weight='bold', color='darkslategrey')

 plt.yticks([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0], fontsize=8, weight='bold', color='darkslategrey')

 plt.show()


def ROC(fpr, tpr, roc_auc):

 legend_properties = {'weight': 'bold', 'size': 8}

 plt.title('Test Data - Receiver Operating Characteristic', fontsize=10, weight='bold', color='steelblue')

 plt.plot(fpr, tpr, 'b', label='AUC = %0.2f' % roc_auc)

 plt.legend(loc='lower right', prop=legend_properties)

 plt.plot([0, 1], [0, 1], 'k--')

 plt.xticks([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0], fontsize=8, weight='bold', color='darkslategrey')

 plt.yticks([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0], fontsize=8, weight='bold', color='darkslategrey')

 plt.ylabel('True Positive Rate', fontdict=fontdict, color='black')

 plt.xlabel('False Positive Rate', fontdict=fontdict, color='black')

 plt.show()


def PR(recall, precision, auc):

 legend_properties = {'weight': 'bold', 'size': 8}

 plt.title('Test Data - Precision-Recall Curve', fontsize=10, weight='bold', color='steelblue')

 plt.plot(recall, precision, 'b', label='AUC = %0.2f' % auc)

 plt.legend(loc='lower right', prop=legend_properties)

 plt.plot([0, 1], [0, 1], 'r--')

 plt.xticks([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0], fontsize=8, weight='bold', color='darkslategrey')

 plt.yticks([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0], fontsize=8, weight='bold', color='darkslategrey')

 plt.ylabel('Precision', fontdict=fontdict, color='black')

 plt.xlabel('Recall', fontdict=fontdict, color='black')

 plt.show()


from distutils.core import setup

setup(
    name='pyspark_easy',  # How you named your package folder (MyLib)
    packages=['pyspark_easy'],  # Chose the same as "name"
    version='1.5',  # Start with a small number and increase it with every change you make
    license='MIT',  # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    description='Makes pyspark dataframe exploration easy',  # Give a short description about your library
    author='Nareshkumar Jayavelu',  # Type in your name
    author_email='nareshkumarj90@gmail.com',  # Type in your E-Mail
    url='https://github.com/naresh-datanut/pyspark-easy',  # Provide either the link to your github or to your website
    download_url='https://github.com/naresh-datanut/pyspark-easy/archive/1.5.tar.gz',  # I explain this later on
    keywords=['pyspark', 'dataframe', 'evaluation', 'model', 'classification', 'multiclass classification',
              'binary classification', 'results', 'summary', 'explore', 'EDA'],
    # Keywords that define your package best
    install_requires=['pyspark',
                      'texttable',
                      'DateTime',
                      'python-dateutil',
                      'numpy',
                      'seaborn',
                      'matplotlib',
                      'pandas',
                      'scikit-plot'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',  # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',  # Again, pick a license
        'Programming Language :: Python :: 3',  # Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)
